#!/usr/bin/env node

/**
 * step1：输入工程名称、模板地址、公共仓库分支、开发端口、预览端口、应用编码、基座地址
 * step2：创建工程、下载模板
 * step3：修改开发端口、预览端口、应用编码
 * step4：拉取公共仓库分支、安装应用依赖
 * step5：根据基座地址修改基座配置
 */

import inquirer from 'inquirer'
import fs from 'fs'
import download from 'download-git-repo'
import ora from 'ora'
import path from 'path'
import shell from 'shelljs'
 
const destDir = process.cwd()
const destFolder = destDir.replace(/\\/g, '/').split('/')
const folderName = destFolder[destFolder.length - 1]
 
// 'spms-example-web-commandcenter'转'exampleCommandcenterApp'
function getAppName(answers) {
    const names = answers.name.split('-')
    const lastName = names[names.length - 1]
    return names[1] + lastName.substring(0, 1).toUpperCase() + lastName.substring(1, lastName.length) + 'App'
}

// 删除目录及文件
async function rmdir(dir) {
    const statObj = await fs.promises.stat(dir)
    if (statObj.isFile()) {
        await fs.promises.unlink(dir)
    } else {
        const dirs = await fs.promises.readdir(dir)
        await Promise.all(dirs.map((item) => rmdir(path.join(dir, item))))
        await fs.promises.rmdir(dir)
    }
}

// 修改模板变量
function modifyTemplate(dir, answers) {
    new Promise((resolve, reject) => {
        fs.readFile(dir, 'utf-8', (err, content) => {
            // content为读取到文件内容
            if (err) throw err
            resolve(content)
        })
    }).then(
        content => {
            // 重写文件内容
            const newContent = content
                .replace(new RegExp('%projectName%', 'g'), answers.name)
                .replace(new RegExp('%projectNameCn%', 'g'), answers.nameCn)
                .replace(new RegExp('%appName%', 'g'), getAppName(answers))
                .replace(new RegExp('%devPort%', 'g'), answers.devPort)
                .replace(new RegExp('%previewPort%', 'g'), answers.previewPort)
                .replace(new RegExp('%appCode%', 'g'), answers.appCode)
            fs.writeFile(dir, newContent, (err) => {
                if (err) throw err
            })
        },
        err => {
            console.log(err)
        }
    )
}

// 修改模板
function modifyFile(answers) {
    const baseUrl = answers.name ? destDir + '/' +  answers.name : destDir
    const list = [
        `${baseUrl}/package.json`,
        `${baseUrl}/app.config.ts`,
        `${baseUrl}/.env.preview`,
        `${baseUrl}/.env.production`,
        `${baseUrl}/.gitignore`,
        `${baseUrl}/blueprint.yaml`,
        `${baseUrl}/README.md`,
        `${baseUrl}/src/router/index.ts`,
        `${baseUrl}/src/router/routesMap.ts`
    ]
    list.forEach(file => {
        modifyTemplate(file, answers)
    })
}

// 修改基座
function modifyBaseApp(answers) {
    // app.config.ts
    fs.readFile(`${answers.baseUrl}/app.config.ts`, 'utf-8', (readErr, content) => {
        // content为读取到文件内容
        if (readErr) throw err
        const [start, end] = content.split('// 更多应用')
        const newContent = `${start}// ${answers.nameCn || answers.name}
        '${answers.name}': {
            dev: {
                url: 'http://localhost:${answers.devPort}/'
            },
            preview: {
                url: 'http://localhost:8081/${answers.name}/'
            },
            build: {
                url: '/${answers.name}/'
            },
            appCode: '${answers.appCode}'
        },
        // 更多应用${end}`
        fs.writeFile(`${answers.baseUrl}/app.config.ts`, newContent, (writeErr) => {
            if (writeErr) throw writeErr
        })
    })
    // src/router/index.ts
    fs.readFile(`${answers.baseUrl}/src/router/index.ts`, 'utf-8', (readErr, content) => {
        // content为读取到文件内容
        if (readErr) throw readErr
        const [start, end] = content.split('const layoutCommonApp = [')
        const newContent = `${start}const layoutCommonApp = [\n    '${answers.name}',${end}`
        fs.writeFile(`${answers.baseUrl}/src/router/index.ts`, newContent, (writeErr) => {
            if (writeErr) throw writeErr
        })
    })
    // src/store/modules/userMenus.ts
    fs.readFile(`${answers.baseUrl}/src/store/modules/userMenus.ts`, 'utf-8', (readErr, content) => {
        // content为读取到文件内容
        if (readErr) throw readErr
        const [start, end] = content.split('const pathPrefixMap = {')
        const newContent = `${start}const pathPrefixMap = {\n        '${answers.appCode}': '/${answers.name}',${end}`
        fs.writeFile(`${answers.baseUrl}/src/store/modules/userMenus.ts`, newContent, (writeErr) => {
            if (writeErr) throw writeErr
        })
    })
}

// 接着通过命令行交互询问用户问题，并获取用户执行结果
inquirer.prompt([
    {
        type: 'input',
        name: 'name',
        message: '请输入项目名称(若为空则当前目录为项目名)', // 询问项目名称并以此名称创建文件夹
        validate (val) {
            if (!val || val.startsWith('spms-')) {
                return true
            }
            return '格式不正确(以"spms-"开头,格式如"spms-wisdom-web-pc")'
        }
    },
    {
        type: 'input',
        name: 'nameCn',
        message: '请输入项目描述(如：智慧应用)' // 询问项目中文名
    },
    {
        type: 'input',
        name: 'templateUrl',
        message: '请输入项目模板gitLab地址', // 询问项目gitlab模板地址
        default: 'git@10.88.40.177:spms-framework/spms-web-pc-template.git#main'
    },
    {
        type: 'input',
        name: 'commonBranch',
        message: '请输入公共代码默认分支', // 公共仓库分支
        default: 'qiankun'
    },
    {
        type: 'input',
        name: 'devPort',
        message: '请输入本地开发端口', // 本地开发服务端口
        validate (val) {
            if (/^[1-9]\d{0,5}$/.test(val)) {
                return true
            }
            return '格式不正确(1-999999数字)'
        }
    },
    {
        type: 'input',
        name: 'previewPort',
        message: '请输入本地预览端口', // 本地预览服务端口
        default: (answers) => {
            return Number(answers.devPort) + 1
        },
        validate (val) {
            if (/^[1-9]\d{0,5}$/.test(val)) {
                return true
            }
            return '格式不正确(1-999999数字)'
        }
    },
    {
        type: 'input',
        name: 'appCode',
        message: '请输入应用编码', // 微应用编码
        default: (answers) => {
            return  (answers.name || folderName).toUpperCase()
        }
     },
    {
        type: 'list',
        name: 'changeBase',
        message: '是否修改基座', // 是否修改基座
        choices: [
            { value: 'true', name: '是' },
            { value: 'false', name: '否' }
        ],
        default: 'false'
    },
    {
        type: 'input',
        name: 'baseUrl',
        message: '本地基座目录', // 本地基座目录
        validate (val) {
            if (!!val) {
                return true
            }
            return '本地基座目录不能为空'
        },
        when: (answers) => {
            return answers.changeBase == 'true'
        }
    },
    {
        type: 'list',
        name: 'tool',
        message: '选择包管理工具', // 选择包管理工具
        choices: [
            { value: 'npm', name: 'npm' },
            { value: 'cnpm', name: 'cnpm' },
            { value: 'yarn', name: 'yarn' },
            { value: 'pnpm', name: 'pnpm' },
            { value: 'none', name: '我不需要安装' }
        ],
        default: 'none'
    },
]).then((answers) => {
    if (!fs.existsSync(answers.name)) {
        const spinner = ora('下载模板...')
        spinner.start()
        // 调用下载方法，传入项目地址，项目名称
        download(`direct:${answers.templateUrl}`, (answers.name || folderName), {clone: true}, (err) => {
            if (err) {
                console.log(err)
                spinner.fail()
            } else {
                console.log('下载成功')
                spinner.succeed()
                // 如果是下载到当前目录，则需移动模板(先复制模板文件/文件夹，再移除原缓存的文件夹)
                if (!answers.name) {
                    // 复制目录
                    fs.cp('./' + folderName, './', { recursive: true }, (err) => {
                        if (err) {
                            console.log('复制失败')
                        } else {
                            console.log('复制成功')
                            rmdir('./' + folderName).then(() => {
                                console.log('删除成功')
                                modifyFile(answers)
                                // 调整基座配置
                                answers.baseUrl && modifyBaseApp(answers)
                                // 安装依赖
                                if (answers.tool !== 'none') {
                                    setTimeout(() => {
                                        // 绑定子仓库
                                        shell.exec(`git submodule add -b ${answers.commonBranch} git@10.88.40.177:spms-framework/spms-web-common.git src/spms-web-common`)
                                        // 拉取子仓库代码
                                        shell.exec(`git submodule init && git submodule update`)
                                        shell.exec(`${answers.tool} install`)
                                    }, 500)
                                }
                            })
                        }
                    })
                } else {
                    modifyFile(answers)
                    // 调整基座配置
                    answers.baseUrl && modifyBaseApp(answers)
                    // 安装依赖
                    if (answers.tool !== 'none') {
                        setTimeout(() => {
                            shell.cd(answers.name)
                            // 绑定子仓库
                            shell.exec(`git submodule add -b ${answers.commonBranch} git@10.88.40.177:spms-framework/spms-web-common.git src/spms-web-common`)
                            // 拉取子仓库代码
                            shell.exec(`git submodule init && git submodule update`)
                            shell.exec(`${answers.tool} install`)
                        }, 500)
                    }
                }
            }
        })
    } else {
        console.log('项目已经存在了')
    }
})